@@ -39,21 +39,21 @@ bidi_subscribeok_test()->
 </presence>"),
     Resp=list_to_binary("<presence to='tintin@jabber.org' type='subscribed'/>"),
     State=#state_rcv{},
-    ?assertMatch({Resp,State}, ts_jabber:parse_bidi(Req,State)).
+    ?assertMatch({Resp,State,think}, ts_jabber:parse_bidi(Req,State)).
 
 bidi_multisubscribeok_test()->
     myset_env(),
     Req=list_to_binary("<presence type='subscribe' to='toto@im.apinc.org' from='tintin@jabber.org'>  <status>Hi dude.</status></presence><presence type='subscribe' to='toto@im.apinc.org' from='glop@jabber.org'>  <status>Copaing?.</status></presence>"),
     Resp=list_to_binary("<presence to='tintin@jabber.org' type='subscribed'/><presence to='glop@jabber.org' type='subscribed'/>"),
     State=#state_rcv{},
-    ?assertMatch({Resp,State}, ts_jabber:parse_bidi(Req,State)).
+    ?assertMatch({Resp,State,think}, ts_jabber:parse_bidi(Req,State)).
 
 bidi_multisubscribe_nok_test()->
     myset_env(),
     Req=list_to_binary("<presence type='subscribe' to='toto@im.apinc.org' from='tintin@jabber.org'>  <status>Hi dude.</status></presence><presence type='subscribed'  from='glop@jabber.org'>  <status>Copaing?.</status></presence>"),
     Resp=list_to_binary("<presence to='tintin@jabber.org' type='subscribed'/>"),
     State=#state_rcv{},
-    ?assertMatch({Resp,State}, ts_jabber:parse_bidi(Req,State)).
+    ?assertMatch({Resp,State,think}, ts_jabber:parse_bidi(Req,State)).
 
 bidi_subscribe_nok_test()->
     myset_env(),
@@ -61,13 +61,13 @@ bidi_subscribe_nok_test()->
   <status>Hi dude.</status>
 </presence>"),
     State=#state_rcv{},
-    ?assertMatch({nodata,State}, ts_jabber:parse_bidi(Req,State)).
+    ?assertMatch({nodata,State,think}, ts_jabber:parse_bidi(Req,State)).
 
 bidi_nok_test()->
     myset_env(),
     Req=list_to_binary("<presence from='tintin@jabber.org'><status>Alive.</status></presence>"),
     State=#state_rcv{},
-    ?assertMatch({nodata,State}, ts_jabber:parse_bidi(Req,State)).
+    ?assertMatch({nodata,State,think}, ts_jabber:parse_bidi(Req,State)).
 
 auth_sasl_test()->
     myset_env(),
@@ -138,6 +138,19 @@ get_online_test()->
     {ok,Online}  = ts_user_server:get_online(Id),
     ?assertEqual({1,3,2},{Id,Offline,Online} ).
 
+get_online_user_test()->
+    Server="myserver",
+    ts_user_server_sup:start_user_server(list_to_atom("us_" ++Server)),
+    MyServer = global:whereis_name(list_to_atom("us_"++Server)),
+    ts_user_server:reset(MyServer,100),
+    Id=ts_user_server:get_idle(MyServer),
+    IdOther = ts_user_server:get_idle(MyServer),
+    RealId  = ts_jabber_common:set_id(IdOther,"tsung","tsung"),
+    ts_user_server:add_to_online(MyServer,RealId),
+    {ok,Offline} = ts_user_server:get_offline(MyServer),
+    {ok,Online}  = ts_user_server:get_online(MyServer, Id),
+    ?assertEqual({1,3,2},{Id,Offline,Online} ).
+
 get_online_user_defined_test()->
     ts_user_server:reset(0),
     ts_msg_server:stop(),
