@@ -50,6 +50,7 @@
          userid/1]).
 
 -export ([starttls_bidi/2,
+          message_bidi/2,
           presence_bidi/2]).
 
 %%----------------------------------------------------------------------
@@ -165,6 +166,7 @@ parse_bidi(Data,  State) ->
     RcvdXml = binary_to_list(Data),
     BidiElements =
         [{"<presence[^>]*subscribe[\"\']", presence_bidi},
+         {"@@@([^@]+)@@@", message_bidi},
          {"<proceed", starttls_bidi}],
     lists:foldl(fun({Regex, Handler}, Acc)->
        case re:run(RcvdXml,Regex) of
@@ -174,13 +176,29 @@ parse_bidi(Data,  State) ->
         _Else ->
             Acc
         end
-    end, {nodata, State}, BidiElements).
+    end, {nodata, State, think}, BidiElements).
 
 presence_bidi(RcvdXml, State)->
     {match,SubMatches} = re:run(RcvdXml,"<presence[^>]*subscribe[\"\'][^>]*>",[global]),
     bidi_resp(subscribed,RcvdXml,SubMatches,State).
 
-starttls_bidi(_RcvdXml, #state_rcv{socket= Socket}=State)->
+message_bidi(RcvdXml, State) ->
+    {match, [NodeStamp]} = re:run(RcvdXml, "@@@([^@]+)@@@", [{capture, all_but_first, list}]),
+    [NodeS, StampS] = string:tokens(NodeStamp, ","),
+    case integer_to_list(erlang:phash2(node())) of
+        NodeS ->
+            [MegaS, SecsS, MicroS] = string:tokens(StampS, ";"),
+            Mega = list_to_integer(MegaS),
+            Secs = list_to_integer(SecsS),
+            Micro = list_to_integer(MicroS),
+            Latency = timer:now_diff(?NOW, {Mega, Secs, Micro}),
+            ts_mon:add({ sample, xmpp_msg_latency, Latency / 1000});
+        _ ->
+            ignore
+    end,
+    {nodata, State, think}.
+
+starttls_bidi(_RcvdXml, #state_rcv{socket= Socket, send_timestamp=SendTime}=State)->
     ssl:start(),
     Req = subst(State#state_rcv.request#ts_request.param, State#state_rcv.dynvars),
     Opt = lists:filter(fun({_,V}) -> V /= undefined end, 
@@ -190,7 +208,9 @@ starttls_bidi(_RcvdXml, #state_rcv{socket= Socket}=State)->
                        {cacertfile,Req#jabber.cacertfile}]),
     {ok, SSL} = ts_ssl:connect(Socket, Opt),
     ?LOGF("Upgrading to TLS : ~p",[SSL],?INFO),
-    {nodata, State#state_rcv{socket=SSL,protocol=ts_ssl}}.
+    Latency = ts_utils:elapsed(SendTime, ?NOW),
+    ts_mon:add({ sample, xmpp_starttls, Latency}),
+    {nodata, State#state_rcv{socket=SSL,protocol=ts_ssl}, continue}.
 
 %%----------------------------------------------------------------------
 %% Function: bidi_resp/4
@@ -201,6 +221,7 @@ starttls_bidi(_RcvdXml, #state_rcv{socket= Socket}=State)->
 %%          State (record)
 %% Returns:    Data (binary)
 %%             NewState (record)
+%%             think|continue
 %%----------------------------------------------------------------------
 %% subscribed: Complete a pending subscription request
 bidi_resp(subscribed,RcvdXml,SubMatches,State) ->
@@ -220,10 +241,10 @@ bidi_resp(subscribed,RcvdXml,SubMatches,State) ->
     end,"",SubMatches),
     case lists:flatten(JoinedXml) of
         "" ->
-            {nodata,State};
+            {nodata,State, think};
         _ ->
             ?LOGF("RESPONSE TO SEND : ~s~n",[JoinedXml],?DEB),
-            {list_to_binary(JoinedXml),State}
+            {list_to_binary(JoinedXml),State, think}
     end.
 
 %%
@@ -269,7 +290,8 @@ add_dynparams2(true, DynVars, Param, _Host) ->
 %% than use side effects, as get_message/1 andn add_dynparams/4 aren't allowed
 %% to return a new DynData, and so they can't modify the session state.
 choose_domain(VHostFileId) ->
-    {ok,Domain} = ts_file_server:get_random_line(VHostFileId),
+    {ok,DomainBin} = ts_file_server:get_random_line(VHostFileId),
+    Domain=binary_to_list(DomainBin),
     UserServer = global:whereis_name(list_to_atom("us_"++Domain)),
     {Domain,UserServer}.
 
