@@ -80,7 +80,7 @@ dump(A,B)->
     ts_plugin:dump(A,B).
 
 parse_bidi(A, B) ->
-    ts_plugin:dump(A,B).
+    ts_plugin:parse_bidi(A,B).
 
 %%----------------------------------------------------------------------
 %% Function: parse/2
@@ -115,7 +115,7 @@ parse_packets(State,Asn1St) ->
         {none,NewAsn1St} ->
             {State#state_rcv{ack_done=false,session=NewAsn1St},[],false};
         {packet,Packet,NewAsn1St} ->
-            {ok,Resp} = asn1rt:decode('ELDAPv3', 'LDAPMessage', Packet),
+            {ok,Resp} = 'ELDAPv3':decode('LDAPMessage', Packet),
             parse_packet(Resp,State#state_rcv{session = NewAsn1St})
     end.
 
