@@ -1,6 +1,6 @@
 -module(ts_ssl).
 
--export([ connect/3,connect/2, send/3, close/1, set_opts/2, protocol_options/1, normalize_incomming_data/2 ]).
+-export([ connect/2, connect/3, connect/4, send/3, close/1, set_opts/2, protocol_options/1, normalize_incomming_data/2 ]).
 
 -behaviour(gen_ts_transport).
 
@@ -8,18 +8,28 @@
 -include("ts_config.hrl").
 
 
-protocol_options(#proto_opts{ssl_ciphers=negociate, certificate = Cert}) ->
-    [binary, {active, once} ] ++ Cert;
+protocol_options(#proto_opts{ssl_ciphers=Ciphers, certificate = Cert,
+                             is_first_connect = First, reuse_sessions =Reuse}) when First or not Reuse->
+    [binary, {active, once}, {reuse_sessions, false} ] ++ Cert ++ set_ciphers(Ciphers);
 protocol_options(#proto_opts{ssl_ciphers=Ciphers, certificate = Cert}) ->
-    ?DebugF("cipher is ~p~n",[Ciphers]),
-    [binary, {active, once}, {ciphers, Ciphers} ] ++ Cert.
+    [binary, {active, once}] ++ Cert ++ set_ciphers(Ciphers).
+
+set_ciphers(negociate)-> [];
+set_ciphers(Ciphers)  -> [{ciphers, Ciphers}].
 
 %% -> {ok, Socket}
-connect(Host, Port, Opts) ->
-    ssl:connect(Host, Port, opts_to_tcp_opts(Opts)).
+connect(Host, Port, Opts) when is_list(Host) ->
+    connect(Host, Port, opts_to_tcp_opts(Opts), infinity);
+
+connect(Socket, Opts, ConnectTimeout) ->
+    ssl:connect(Socket, opts_to_tcp_opts(Opts), ConnectTimeout).
+
+connect(Host, Port, Opts, ConnectTimeout) ->
+    ssl:connect(Host, Port, opts_to_tcp_opts(Opts), ConnectTimeout).
+
+connect(Socket, Opts) ->
+    connect(Socket, Opts, infinity).
 
-connect(Socket, Opts)->
-    ssl:connect(Socket,  opts_to_tcp_opts(Opts)).
 
 opts_to_tcp_opts(Opts) -> Opts.
 
