@@ -26,7 +26,7 @@
 -vc('$Id$ ').
 -author('nicolas.niclausse@niclux.org').
 
--include("ts_profile.hrl").
+-include("ts_macros.hrl").
 
 %% to get file_info record definition
 -include_lib("kernel/include/file.hrl").
@@ -34,17 +34,19 @@
 %% user interface
 -export([debug/3, debug/4, get_val/1, init_seed/0, chop/1, elapsed/2,
          now_sec/0, node_to_hostname/1, add_time/2, keyumerge/3, key1search/2,
-         level2int/1, mkey1search/2, close_socket/2, datestr/0, datestr/1,
+         level2int/1, mkey1search/2, datestr/0, datestr/1, size_or_length/1,
          erl_system_args/0, erl_system_args/1, setsubdir/1, export_text/1,
          foreach_parallel/2, spawn_par/3, inet_setopts/3, resolve/2,
          stop_all/2, stop_all/3, stop_all/4, join/2, split/2, split2/2, split2/3,
-         make_dir_rec/1, is_ip/1, from_https/1, to_https/1, keymax/2,
+         make_dir/1, make_dir_raw/1, is_ip/1, from_https/1, to_https/1, keymax/2,
          check_sum/3, check_sum/5, clean_str/1, file_to_list/1, term_to_list/1,
-         decode_base64/1, encode_base64/1, to_lower/1, release_is_newer_or_eq/1,
+         decode_base64/1, encode_base64/1, to_lower/1,
          randomstr/1,urandomstr/1,urandomstr_noflat/1, eval/1, list_to_number/1,
-         time2sec/1, time2sec_hires/1, read_file_raw/1, init_seed/1, jsonpath/2, pmap/2,
-         concat_atoms/1, ceiling/1, accept_loop/3, append_to_filename/3, splitchar/2
-        ]).
+         time2sec/1, time2sec_hires/1, read_file_raw/1, init_seed/1, jsonpath/2,
+         concat_atoms/1, ceiling/1, accept_loop/3, append_to_filename/3, splitchar/2,
+         randombinstr/1,urandombinstr/1,log_transaction/1,conv_entities/1, wildcard/2,
+         ensure_all_started/2, pmap/2, pmap/3, get_node_id/0, filtermap/2, new_ets/2,
+         is_controller/0]).
 
 level2int("debug")     -> ?DEB;
 level2int("info")      -> ?INFO;
@@ -61,6 +63,8 @@ level2int("emergency") -> ?EMERG.
 -define(LT,"&lt;").
 -define(DUPSTR_SIZE,20).
 -define(DUPSTR,"qxvmvtglimieyhemzlxc").
+-define(DUPBINSTR_SIZE,20).
+-define(DUPBINSTR,<<"qxvmvtglimieyhemzlxc">>).
 
 %%----------------------------------------------------------------------
 %% Func: get_val/1
@@ -89,6 +93,8 @@ ensure_string(proxy_log_file, Atom) when is_atom(Atom) ->
     atom_to_list(Atom);
 ensure_string(config_file, Atom) when is_atom(Atom) ->
     atom_to_list(Atom);
+ensure_string(exclude_tag, Atom) when is_atom(Atom) ->
+    atom_to_list(Atom);
 ensure_string(_, Other) ->
     Other.
 
@@ -117,7 +123,11 @@ debug(From, Message, Args, Level) ->
 elapsed({Before1, Before2, Before3}, {After1, After2, After3}) ->
     After  = After1  * 1000000000  + After2  * 1000 + After3/1000,
     Before = Before1 * 1000000000  + Before2 * 1000 + Before3/1000,
-    After - Before.
+    case After - Before of
+        Neg when Neg < 0 -> % time duration must not be negative
+             0;
+        Val -> Val
+    end.
 
 %%----------------------------------------------------------------------
 %% Func: chop/1
@@ -157,17 +167,28 @@ init_seed({A,B}) when is_integer(A) and is_integer(B)->
     %% initial pseudo random values will be quite closed to each
     %% other. Trying to avoid this by using a multiplier big enough
     %% (because the algorithm use mod 30XXX , see random.erl).
-    random:seed(1000*A*A,-1000*B*B,1000*Id*Id);
+    random:seed(4000*A*B*Id,-4000*B*A*Id,4000*Id*Id*A);
 init_seed({A,B,C}) ->
     random:seed(A,B,C).
 
 get_node_id() ->
     case string:tokens(atom_to_list(node()),"@") of
         ["tsung_control"++_,_]    -> 123456;
-        ["tsung"++I,_]            -> list_to_integer(I);
+        ["tsung"++Tail,_]         ->
+            {match, [I]} = re:run(Tail, "\\d+$", [{capture, all, list}]),
+            list_to_integer(I);
         _                         -> 654321
     end.
 
+%% @spec is_controller() -> true|false
+%% @doc return true if the caller is running on the controller node
+%% @end
+is_controller() ->
+    case string:tokens(atom_to_list(node()),"@") of
+        ["tsung_control"++_,_]    -> true;
+        _ ->false
+    end.
+
 %%----------------------------------------------------------------------
 %% Func: init_seed/0
 %%----------------------------------------------------------------------
@@ -195,7 +216,7 @@ add_time({MSec, Seconds, MicroSec}, SecToAdd) when is_integer(SecToAdd)->
     NewSec = Seconds +SecToAdd,
     case NewSec < 1000000 of
         true -> {MSec, NewSec, MicroSec};
-        false ->{MSec+ (NewSec div 100000), NewSec-1000000, MicroSec}
+        false ->{MSec+ (NewSec div 1000000), NewSec-1000000, MicroSec}
     end.
 
 node_to_hostname(Node) ->
@@ -211,10 +232,18 @@ encode_base64(String)->
 decode_base64(Base64)->
     base64:decode_to_string(Base64).
 
-% return true if current version of erlang is newer or equal
-release_is_newer_or_eq(Release)->
-    erlang:system_info(version) >= Release.
-
+%%----------------------------------------------------------------------
+%% Func: filtermap/2
+%% Purpose lists:zf is called lists:filtermap in erlang R16B1 and newer
+%%
+%%----------------------------------------------------------------------
+filtermap(Fun, List)->
+    case erlang:function_exported(lists, filtermap, 2) of
+        true ->
+            lists:filtermap(Fun,List);
+        false ->
+            lists:zf(Fun,List)
+    end.
 
 %%----------------------------------------------------------------------
 %% Func: key1search/2
@@ -246,15 +275,6 @@ mkey1search(List, Key) ->
         Results -> lists:reverse(Results)
     end.
 
-%% close socket if it exists
-close_socket(_Protocol, none) -> ok;
-close_socket(gen_tcp, Socket) -> gen_tcp:close(Socket);
-close_socket(gen_tcp6, Socket)-> gen_tcp:close(Socket);
-close_socket(ssl, Socket)     -> ssl:close(Socket);
-close_socket(ssl6, Socket)    -> ssl:close(Socket);
-close_socket(gen_udp, Socket) -> gen_udp:close(Socket);
-close_socket(gen_udp6, Socket)-> gen_udp:close(Socket).
-
 %%----------------------------------------------------------------------
 %% datestr/0
 %% Purpose: print date as a string 'YYYYMMDD-HHMM'
@@ -314,13 +334,26 @@ erl_system_args(extended)->
                     undefined -> "";
                     {ok, Max} -> " -kernel inet_dist_listen_max " ++ integer_to_list(Max)++" "
                 end,
+    SSLCache = case application:get_env(ssl,session_cb) of
+                   {ok, CB} when is_atom(CB) -> " -ssl session_cb " ++ atom_to_list(CB)++" ";
+                   _ -> ""
+               end,
+    SSLLifetime = case application:get_env(ssl,session_lifetime) of
+                   {ok, Time} when is_integer(Time) -> " -ssl session_lifetime " ++ integer_to_list(Time)++" ";
+                   _ -> ""
+               end,
+    SSLCacheSize = case application:get_env(tsung,ssl_session_cache) of
+                   {ok, Reuse} when is_integer(Reuse)-> " -tsung reuse_sessions " ++ integer_to_list(Reuse)++" ";
+                   _ -> ""
+               end,
     Threads= "+A "++integer_to_list(erlang:system_info(thread_pool_size))++" ",
     ProcessMax="+P "++integer_to_list(erlang:system_info(process_limit))++" ",
     Mea = case  erlang:system_info(version) of
               "5.3" ++ _Tail     -> " +Mea r10b ";
               _ -> " "
           end,
-    lists:append([BasicArgs, Shared, Hybrid, Smp, Mea, Inet, Proto, Threads,ProcessMax,ListenMin,ListenMax]).
+    lists:append([BasicArgs, Shared, Hybrid, Smp, Mea, Inet, Proto, Threads,
+                  ProcessMax,ListenMin,ListenMax,SSLCache,SSLLifetime,SSLCacheSize]).
 
 %%----------------------------------------------------------------------
 %% setsubdir/1
@@ -394,30 +427,38 @@ stop_all(_,_,_,_)->
     erlang:display("Bad Hostname").
 
 %%----------------------------------------------------------------------
-%% make_dir_rec/1
+%% make_dir/1
 %% Purpose: create directory. Missing parent directories ARE created
 %%----------------------------------------------------------------------
-make_dir_rec(DirName) when is_list(DirName) ->
-    case  file:read_file_info(DirName) of
+make_dir(DirName) ->
+    make_dir_rec(DirName,file).
+
+make_dir_raw(DirName) ->
+    make_dir_rec(DirName,prim_file).
+
+make_dir_rec(DirName,FileMod) when is_list(DirName) ->
+    case  FileMod:read_file_info(DirName) of
         {ok, #file_info{type=directory}} ->
             ok;
         {error,enoent} ->
-            make_dir_rec("", filename:split(DirName));
+            make_dir_rec("", FileMod,filename:split(DirName));
         {error, Reason}  ->
             {error,Reason}
     end.
 
-make_dir_rec(_Path, []) ->
+make_dir_rec(_Path, _FileMod, []) ->
     ok;
-make_dir_rec(Path, [Parent|Childs]) ->
+make_dir_rec(Path, FileMod,[Parent|Childs]) ->
     CurrentDir=filename:join([Path,Parent]),
-    case  file:read_file_info(CurrentDir) of
+    case  FileMod:read_file_info(CurrentDir) of
         {ok, #file_info{type=directory}} ->
-            make_dir_rec(CurrentDir, Childs);
+            make_dir_rec(CurrentDir, FileMod,Childs);
         {error,enoent} ->
-            case file:make_dir(CurrentDir) of
+            case FileMod:make_dir(CurrentDir) of
                 ok ->
-                    make_dir_rec(CurrentDir, Childs);
+                    make_dir_rec(CurrentDir, FileMod, Childs);
+                {error, eexist} ->
+                    make_dir_rec(CurrentDir, FileMod, Childs);
                 Error ->
                     Error
             end;
@@ -443,8 +484,8 @@ to_https({url, "http://-"++Rest})-> "https://" ++ Rest;
 to_https({url, URL})-> URL;
 to_https({request, {body,Data}}) when is_list(Data) ->
     %% body request, no headers
-    re:replace(Data,"http://-","https://",[global]);
-to_https({request, S="CONNECT"++Rest}) -> {ok,S};
+    {ok, re:replace(Data,"http://-","https://",[global])};
+to_https({request, S="CONNECT"++_Rest}) -> {ok,S};
 to_https({request, []}) -> {ok, []};
 to_https({request, String}) when is_list(String) ->
     EndOfHeader = string:str(String, "\r\n\r\n"),
@@ -452,8 +493,8 @@ to_https({request, String}) when is_list(String) ->
     Body = string:substr(String, EndOfHeader + 4),
     ReOpts=[global,{return,list}],
     TmpHeader = re:replace(Header,"http://-","https://",ReOpts),
-    TmpHeader2 = re:replace(TmpHeader,"Accept-Encoding: [0-9,a-zA-Z_ ]+\r\n","",ReOpts),
-    RealHeader = re:replace(TmpHeader2,"Host: -","Host: ",ReOpts),
+    TmpHeader2 = re:replace(TmpHeader,"Accept-Encoding: [0-9,a-z_ ]+\r\n","",ReOpts++[caseless]),
+    RealHeader = re:replace(TmpHeader2,"Host: -","Host: ",ReOpts++[caseless]),
     RealBody = re:replace(Body,"http://-","https://",ReOpts),
     RealString = RealHeader++  "\r\n" ++ RealBody,
     {ok, RealString}.
@@ -481,15 +522,19 @@ join(_Sep, []) -> [];
 join(Sep, List) when is_list(List)->
     ToStr = fun(A) when is_integer(A) -> integer_to_list(A);
                (A) when is_list(A) -> A;
-               (A) when is_float(A) -> float_to_list(A);
+               (A) when is_float(A) -> io_lib:format("~.3f",[A]);
                (A) when is_atom(A) -> atom_to_list(A);
                (A) when is_binary(A) -> binary_to_list(A)
             end,
     string:join(lists:map(ToStr,List), Sep).
 
 %% split a string given a string (at first occurence of char)
-split(String,Chr) ->
-    re:split(String,Chr,[{return,list}]).
+split(String,Chr) when is_list(String), is_list(Chr) ->
+    re:split(String,Chr,[{return,list}]);
+
+split(String,Chr) when is_binary(String), is_binary(Chr) ->
+    binary:split(String,[Chr],[global]).
+
 
 %% split a string given a char (faster)
 splitchar(String,Chr) ->
@@ -628,6 +673,7 @@ keyumerge(N,[A|Rest],B)->
 %% Purpose: Return Max of Nth element of a list of tuples
 %% Returns: Number
 %%----------------------------------------------------------------------
+keymax(_N,[])-> 0;
 keymax(N,[L])-> element(N,L);
 keymax(N,[E|Tail])->
     keymax(N,Tail,element(N,E)).
@@ -675,6 +721,19 @@ urandomstr_noflat(Size) when is_integer(Size) , Size >= ?DUPSTR_SIZE ->
 urandomstr_noflat(Size)  when is_integer(Size), Size >= 0 ->
     lists:nthtail(?DUPSTR_SIZE-Size, ?DUPSTR).
 
+%%----------------------------------------------------------------------
+%% @spec urandombinstr(Size::integer()) ->binary()
+%% @doc same as urandomstr/1, but returns a binary.
+%% @end
+%%----------------------------------------------------------------------
+urandombinstr(Size) when is_integer(Size) , Size >= ?DUPBINSTR_SIZE ->
+    Loop = Size div ?DUPBINSTR_SIZE,
+    Rest = Size rem ?DUPBINSTR_SIZE,
+    Res=lists:foldl(fun(_X,Acc)-> <<Acc/binary, ?DUPBINSTR/binary>> end, << >>,lists:seq(1,Loop)),
+    << Res/binary, ?DUPBINSTR:Rest/binary>>;
+urandombinstr(Size) when is_integer(Size), Size >= 0 ->
+    <<?DUPBINSTR:Size/binary>> .
+
 %%----------------------------------------------------------------------
 %% @spec urandomstr(Size::integer()) ->string()
 %% @doc same as urandomstr_noflat/1, but returns a flat list.
@@ -691,6 +750,19 @@ urandomstr(Size) when is_integer(Size), Size >= 0 ->
 randomstr(Size) when is_integer(Size), Size >= 0 ->
      lists:map(fun (_) -> random:uniform(25) + $a  end, lists:seq(1,Size)).
 
+%%----------------------------------------------------------------------
+%% @spec randombinstr(Size::integer()) ->binary()
+%% @doc returns a random binary string. slow if Size is high.
+%% @end
+%%----------------------------------------------------------------------
+randombinstr(0) -> <<>>;
+randombinstr(Size) when is_integer(Size), Size > 0 ->
+    randombinstr(Size,<<>>).
+randombinstr(0,Bin) -> Bin;
+randombinstr(Size,Bin) ->
+    C=random:uniform(25)+$a,
+    randombinstr(Size-1, << Bin/binary, C >>).
+
 
 %%----------------------------------------------------------------------
 %% @spec eval(string()) -> term()
@@ -763,12 +835,12 @@ jsonpath(JSONPath,JSON) ->
                         list_to_binary(A)
                 end
           end,
-    Str=re:replace(JSONPath,"\\[(.*)\\]","\.\\1",[{return,list},global]),
+    Str=re:replace(JSONPath,"\\[(.*?)\\]","\.\\1",[{return,list},global]),
     Keys=lists:map(Fun, string:tokens(Str,".")),
     json_get_bin(Keys,JSON).
 json_get_bin([],Val) ->
     Val;
-json_get_bin([Key|Keys],undefined) ->
+json_get_bin([_Key|_Keys],undefined) ->
     undefined;
 json_get_bin([N|Keys],L) when is_integer(N), N =< length(L) ->
     Val =  lists:nth(N,L),
@@ -811,6 +883,31 @@ pmap(F, L) ->
     [receive {Pid, Result} -> Result end || Pid <- [spawn(fun() -> Parent ! {self(), F(X)} end) || X <- L]].
 
 
+%% Map function F over list L in parallel, with maximum NProcs in parallel
+%% FIXME: handle timeout
+pmap(F, L, NProcs) ->
+    pmap(F, L, NProcs,"").
+
+pmap(F, L, NProcs, Res) when length(L) > NProcs->
+    {Head, Tail} = lists:split(NProcs,L),
+    Parent = self(),
+    lists:foldl(fun(X, Acc) -> spawn(fun() -> Parent ! {pmap, self(), F(X), Acc} end), Acc+1  end, 0, Head),
+    NewRes = wait_result(NProcs,[]),
+    pmap(F,Tail, NProcs, Res ++ NewRes);
+
+pmap(F, L, _NProcs, Acc) ->
+    Acc ++ pmap(F,L).
+
+wait_result(0, Res)->
+    {_Ids, RealRes} = lists:unzip(lists:keysort(1, Res)),
+    RealRes;
+wait_result(Nprocs, Res)->
+    receive
+        {pmap, _Pid, Result, Id} ->
+            NewRes = Res ++ [{Id, Result}],
+            wait_result(Nprocs-1, NewRes)
+    end.
+
 %%
 ceiling(X) ->
     T = erlang:trunc(X),
@@ -850,3 +947,66 @@ append_to_filename(Filename, From, To) ->
         RealName -> RealName
     end.
 
+
+log_transaction([]) ->
+    "-";
+log_transaction([{TransactionName,_}| _Tail]) ->
+    TransactionName.
+
+%%--------------------------------------------------------------------
+%% Func: conv_entities/1
+%% Purpose: Convert html entities to string
+%%--------------------------------------------------------------------
+conv_entities(Binary)->
+    conv_entities(Binary,[]).
+conv_entities(<< >>,Acc) ->
+    list_to_binary(Acc);
+conv_entities(<< "&amp;", T/binary >> ,Acc) ->
+    conv_entities(T,[ Acc, << "&">>]);
+conv_entities(<< "&lt;", T/binary >>,Acc) ->
+    conv_entities(T,[ Acc, << "<">>]);
+conv_entities(<< "&gt;", T/binary >>,Acc) ->
+    conv_entities(T,[ Acc, << ">">>]);
+conv_entities(<<"&quot;", T/binary >>,Acc) ->
+    conv_entities(T,[ Acc, << "\"">>]);
+conv_entities(<<"&apos;", T/binary >>,Acc) ->
+    conv_entities(T,[ Acc, << "'">>]);
+conv_entities(<<H:1/binary, T/binary >>,Acc) ->
+    conv_entities(T,[ Acc, H]).
+
+%% start an application and it's dependencies recursively
+%% does the same as application:ensure_all_started (only in R16B2)
+ensure_all_started(App, Type) ->
+    start_ok(App, Type, application:start(App, Type)).
+
+start_ok(_App, _Type, ok) -> ok;
+start_ok(_App, _Type, {error, {already_started, _App}}) -> ok;
+start_ok(App, Type, {error, {not_started, Dep}}) ->
+    ok = ensure_all_started(Dep, Type),
+    ensure_all_started(App, Type);
+start_ok(App, _Type, {error, Reason}) ->
+    erlang:error({app_start_failed, App, Reason}).
+
+wildcard(Wildcard,Names) ->
+    PatternTmp = re:replace("^"++Wildcard,"\\*",".*",[{return,list}]),
+    Pattern = re:replace(PatternTmp,"\\?",".{1}",[{return,list}]) ++ "$" ,
+    lists:filter(fun(N) -> re:run(N, Pattern) =/= nomatch end, Names).
+
+%%--------------------------------------------------------------------
+%% Func: new_ets/1
+%% Purpose: Wrapper for ets:new/1 used in external modules
+%% @spec new_ets(Prefix::binary(), UserId::integer()) -> string()
+%% @doc init an ets:table
+%% @end
+%%--------------------------------------------------------------------
+new_ets(Prefix, UserId)->
+    EtsName = binary_to_list(Prefix) ++ "_" ++ integer_to_list(UserId),
+    ?LOGF("create ets:table ~p ~n", [EtsName], ?INFO),
+    ets:new(list_to_atom(EtsName), []).
+
+
+size_or_length(Data) when is_binary(Data) ->
+    size(Data);
+size_or_length(Data) when is_list(Data) ->
+    length(Data).
+    
