@@ -19,7 +19,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 %%%----------------------------------------------------------------------
 %%% @author Nicolas Niclausse <nicolas.niclausse@niclux.org>
@@ -46,6 +47,7 @@
          getAttr/4,
          getText/1,
          parse/2,
+         get_default/2,
          get_default/3,
          mark_prev_req/3,
          get_batch_nodes/1
@@ -65,9 +67,17 @@ read(Filename=standard_io, LogDir) ->
                                        {validation,true}]),Filename,LogDir);
 read(Filename, LogDir) ->
     ?LOGF("Reading config file: ~s~n", [Filename], ?NOTICE),
-    handle_read(catch xmerl_scan:file(Filename,
-                                      [{fetch_path,["/usr/share/tsung/","./"]},
-                                       {validation,true}]),Filename,LogDir).
+    Result = handle_read(catch xmerl_scan:file(Filename,
+                                               [{fetch_path,["/usr/share/tsung/","./"]},
+                                                {validation,true}]),Filename,LogDir),
+    %% In case of error we reparse the file with xmerl_sax_parser:file/2 to obtain
+    %% a more verbose output
+    case Result of
+        {ok, _} -> Result;
+        _ -> xmerl_sax_parser:file(Filename,[]),
+             Result
+    end.
+
 
 handle_read( {Root = #xmlElement{}, _Tail}, Filename, LogDir) ->
     Table = ets:new(sessiontable, [ordered_set, protected]),
@@ -90,7 +100,8 @@ parse(Element = #xmlElement{parents = [], attributes=Attrs}, Conf=#config{}) ->
                    "false" -> none;
                    "true"  -> full;
                    "light" -> light;
-                   "protocol" -> protocol
+                   "protocol" -> protocol;
+                   "protocol_local" -> protocol_local
                end,
     lists:foldl(fun parse/2,
                 Conf#config{dump= DumpType, stats_backend=BackEnd,
@@ -99,16 +110,18 @@ parse(Element = #xmlElement{parents = [], attributes=Attrs}, Conf=#config{}) ->
 
 
 %% parsing the Server elements
-parse(Element = #xmlElement{name=server, attributes=Attrs}, Conf=#config{servers=ServerList}) ->
+parse(Element = #xmlElement{name=server, attributes=Attrs}, Conf=#config{servers=ServerList, total_server_weights=OldTotal}) ->
     Server = getAttr(Attrs, host),
     Port   = getAttr(integer, Attrs, port),
-    Type = set_net_type(getAttr(Attrs, type)),
-
+    Weight = getAttr(float_or_integer, Attrs, weight,1),
+    Type   = set_net_type(getAttr(Attrs, type)),
+    Total  = OldTotal + Weight,
     lists:foldl(fun parse/2,
-        Conf#config{servers = [#server{host=Server,
-                                       port=Port,
-                                       type=Type
-                                     }|ServerList]},
+        Conf#config{servers = [#server{host  = Server,
+                                       port  = Port,
+                                       weight= Weight,
+                                       type  = Type
+                                     }|ServerList], total_server_weights = Total},
         Element#xmlElement.content);
 
 
@@ -119,7 +132,19 @@ parse(Element = #xmlElement{name=monitor, attributes=Attrs},
     Host = getAttr(Attrs, host),
     Type = case getAttr(atom, Attrs, type, erlang) of
                erlang ->
-                   {erlang, {}};
+                   case lists:keysearch(mysqladmin,#xmlElement.name,
+                                        Element#xmlElement.content) of
+                       {value, MysqlEl=#xmlElement{} } ->
+                           Port = getAttr(integer,MysqlEl#xmlElement.attributes,
+                                                    port, ?config(mysql_port)),
+                           Username = getAttr(string,MysqlEl#xmlElement.attributes,
+                                                    username, ?config(mysql_user)),
+                           Password = getAttr(string,MysqlEl#xmlElement.attributes,
+                                                    password, ?config(mysql_password)),
+                           {erlang, [{mysqladmin, {Port, Username, Password}}]};
+                        _ ->
+                           {erlang, []}
+                    end;
                snmp ->
                    case lists:keysearch(snmp,#xmlElement.name,
                                         Element#xmlElement.content) of
@@ -211,22 +236,24 @@ parse(Element = #xmlElement{name=client, attributes=Attrs},
                 DeleteController=fun(A) when A == ControllerHost -> false;
                                     (_) -> true
                                  end,
-                Nodes = lists:filter(DeleteController, NodesTmp),
+                Nodes = case lists:filter(DeleteController, NodesTmp) of
+                            []  -> NodesTmp; %% all nodes are on the controller, don't remove them
+                            Val -> Val
+                        end,
                 Fun = fun(N)->
                               IP = case Scan_Intf of
                                        "" ->
-                                           {ok, TmpIP } = inet:getaddr(N,inet),
-                                           TmpIP;
+                                           [];
                                        Interface ->
                                            case os:type() of
                                                {unix, linux} ->
-                                                   {scan, Interface};
+                                                    [{scan, Interface}];
                                                OS ->
                                                    io:format(standard_error,"Scan interface is not supported on OS ~p, abort~n",[OS]),
                                                    exit({error, scan_interface_not_supported_on_os})
                                            end
                                    end,
-                              #client{host=N,weight=Weight,ip=[IP],maxusers=MaxUsers}
+                              #client{host=N,weight=Weight,ip=IP,maxusers=MaxUsers}
                       end,
                 lists:map(Fun, Nodes);
             _ ->
@@ -286,7 +313,7 @@ parse(Element = #xmlElement{name=ip, attributes=Attrs},
 parse(Element = #xmlElement{name=arrivalphase, attributes=Attrs},
       Conf = #config{arrivalphases=AList}) ->
 
-    Phase     = getAttr(integer,Attrs, phase),
+    Phase      = getAttr(integer,Attrs, phase),
     IDuration  = getAttr(integer, Attrs, duration),
     Unit  = getAttr(string,Attrs, unit, "second"),
     D = to_milliseconds(Unit, IDuration),
@@ -357,8 +384,10 @@ parse(Element = #xmlElement{name=session, attributes=Attrs},
     Bidi        = getAttr(atom,Attrs, bidi, Bidi_def),
     Name        = getAttr(Attrs, name),
     ?LOGF("Session name for id ~p is ~p~n",[Id+1, Name],?NOTICE),
-    ?LOGF("Session type: persistent=~p, bidi=~p~n",[Persistent,Bidi],?NOTICE),
-    Probability = getAttr(float_or_integer, Attrs, probability),
+    ?LOGF("Session type: persistent=~p, bidi=~p~n",[Persistent,Bidi],?INFO),
+    Probability = getAttr(float_or_integer, Attrs, probability, -1),
+    Weight      = getAttr(float_or_integer, Attrs, weight, -1),
+    {Popularity, NewUseWeights, NewTotal} = get_popularity(Probability, Weight, Conf#config.use_weights,Conf#config.total_popularity),
     NewSList = case SList of
                    [] -> []; % first session
                    [Previous|Tail] ->
@@ -367,7 +396,7 @@ parse(Element = #xmlElement{name=session, attributes=Attrs},
                end,
     lists:foldl(fun parse/2,
                 Conf#config{sessions = [#session{id           = Id + 1,
-                                                 popularity   = Probability,
+                                                 popularity   = Popularity,
                                                  type         = Type,
                                                  name         = Name,
                                                  persistent   = Persistent,
@@ -378,9 +407,27 @@ parse(Element = #xmlElement{name=session, attributes=Attrs},
                                                 }
                                         |NewSList],
                             main_sess_type = Type,
+                            use_weights=NewUseWeights,
+                            total_popularity=NewTotal,
                             curid=0, cur_req_id=0},% re-initialize request id
                 Element#xmlElement.content);
 
+%% Parsing the session_setup element
+parse(Element = #xmlElement{name=session_setup, attributes=Attrs}, Conf = #config{arrivalphases=[Phase|Phases]}) ->
+
+    Name         = getAttr(Attrs, name),
+    {Popularity, NewUseWeights} =  case { Conf#config.use_weights,  getAttr(float_or_integer, Attrs, weight, -1) } of
+                                      {Use, -1} when (Use == undefined orelse Use == false) ->
+                                          { getAttr(float_or_integer, Attrs, probability, -1), false };
+                                      {_, Val} ->
+                                          { Val, true}
+                                  end,
+    SessionsPopularities = Phase#arrivalphase.popularities,
+    NewPhase = Phase#arrivalphase{popularities= [{Name, Popularity}| SessionsPopularities]},
+    lists:foldl(fun parse/2,
+                Conf#config{ use_weights      = NewUseWeights, arrivalphases = [NewPhase | Phases] },
+                Element#xmlElement.content);
+
 %%%% Parsing the transaction element
 parse(Element = #xmlElement{name=transaction, attributes=Attrs},
       Conf = #config{session_tab = Tab, sessions=[CurS|_], curid=Id}) ->
@@ -404,10 +451,25 @@ parse(Element = #xmlElement{name=transaction, attributes=Attrs},
 parse(_Element = #xmlElement{name='if', attributes=Attrs,content=Content},
       Conf = #config{session_tab = Tab, sessions=[CurS|_], curid=Id}) ->
     VarName=get_dynvar_name(getAttr(string,Attrs,var)),
-    {Rel,Value} = case getAttr(string,Attrs,eq,none) of
-                none -> {neq,getAttr(string,Attrs,neq)};
-                X ->  {eq,X}
-            end,
+    {Rel,Value} = case {getAttr(string,Attrs,eq,none),
+                        getAttr(string,Attrs,neq,none),
+                        getAttr(string,Attrs,gt,none),
+                        getAttr(string,Attrs,lt,none),
+                        getAttr(string,Attrs,gte,none),
+                        getAttr(string,Attrs,lte,none)} of
+                      {none, Neq, none, none, none, none} ->
+                          {neq,Neq};
+                      {Eq, none, none, none, none, none} ->
+                          {eq,Eq};
+                      {none, none, Gt, none, none, none} ->
+                          {gt,Gt};
+                      {none, none, none, Lt, none, none} ->
+                          {lt,Lt};
+                      {none, none, none, none, Gte, none} ->
+                          {gt,Gte};
+                      {none, none, none, none, none, Lte} ->
+                          {lt,Lte}
+                  end,
     ?LOGF("Add if_start action in session ~p as id ~p",
           [CurS#session.id,Id+1],?INFO),
     NewConf = lists:foldl(fun parse/2, Conf#config{curid=Id+1}, Content),
@@ -483,11 +545,28 @@ parse(_Element = #xmlElement{name=repeat,attributes=Attrs,content=Content},
     case LastElement of
         #xmlElement{name=While,attributes=WhileAttrs}
         when (While == 'while') or (While == 'until')->
-            {Rel,Value} = case getAttr(string,WhileAttrs,eq,none) of
-                              none -> {neq,getAttr(string,WhileAttrs,neq)};
-                              X ->  {eq,X}
+            {Rel,Value} = case {getAttr(string,WhileAttrs,eq,none),
+                                getAttr(string,WhileAttrs,neq,none),
+                                getAttr(string,WhileAttrs,gt,none),
+                                getAttr(string,WhileAttrs,lt,none),
+                                getAttr(string,WhileAttrs,gte,none),
+                                getAttr(string,WhileAttrs,lte,none)} of
+                              {none, Neq, none, none, none, none} ->
+                                  {neq,Neq};
+                              {Eq, none, none, none, none, none} ->
+                                  {eq,Eq};
+                              {none, none, Gt, none, none, none} ->
+                                  {gt,Gt};
+                              {none, none, none, Lt, none, none} ->
+                                  {lt,Lt};
+                              {none, none, none, none, Gte, none} ->
+                                  {gt,Gte};
+                              {none, none, none, none, none, Lte} ->
+                                  {lt,Lte}
                           end,
-                          %either <while .. eq=".."/> or <while ..neq=".."/>
+                          %either <while .. eq=".."/> , <while ..neq=".."/>
+                          %either <while .. gt=".."/> , <while ..gte=".."/>
+                          %either <while .. lt=".."/> , <while ..lte=".."/>
             Var = getAttr(atom,WhileAttrs,var),
             NewConf = lists:foldl(fun parse/2, Conf#config{curid=Id}, Content),
             NewId = NewConf#config.curid,
@@ -506,35 +585,40 @@ parse(#xmlElement{name=dyn_variable, attributes=Attrs},
       Conf=#config{sessions=[CurS|_],dynvar=DynVars}) ->
     StrName  = ts_utils:clean_str(getAttr(Attrs, name)),
     {ok, [{atom,1,Name}],1} = erl_scan:string("'"++StrName++"'"),
-    {Type,Expr} = case {getAttr(string,Attrs,regexp,none),
-                        getAttr(string,Attrs,re,none),
+    {Type,Expr} = case {getAttr(string,Attrs,re,none),
                         getAttr(string,Attrs,pgsql_expr,none),
                         getAttr(string,Attrs,xpath,none),
+                        getAttr(string,Attrs,header,none),
                        getAttr(string,Attrs,jsonpath,none)} of
                       {none,none,none,none,none} ->
                           DefaultRegExp = ?DEF_RE_DYNVAR_BEGIN ++ StrName
                               ++?DEF_RE_DYNVAR_END,
                           {re,DefaultRegExp};
-                      {none,none,none,XPath,none} ->
-                          {xpath,XPath};
-                      {none,none,none,none,JSONPath} ->
-                          {jsonpath,JSONPath};
-                      {none,none,PG,none,none} ->
-                          {pgsql_expr,PG};
-                      {none,RE,none,none,none} ->
+                      {RE,none,none,none, none} ->
                           {re,RE};
-                      {RegExp,_,_,_,_} ->
-                          {regexp,RegExp}
+                      {none,PG,none,none, none} ->
+                          {pgsql_expr,PG};
+                      {none,none,XPath,none,none} ->
+                          {xpath,XPath};
+                      {none,none,none,AuthHeader,none} ->
+                          {header, AuthHeader};
+                      {none,none,none,none,JSONPath} ->
+                          {jsonpath,JSONPath}
                   end,
     FlattenExpr =lists:flatten(Expr),
     %% precompilation of the exp
     DynVar = case Type of
-                 regexp ->
-                     exit({error, regexp_obsolete_use_re});
                  re ->
                      ?LOGF("Add new re: ~s ~n", [Expr],?INFO),
                      {ok, CompiledRegExp} = re:compile(FlattenExpr),
-                     {re,Name,CompiledRegExp};
+                     %% TSUN-249
+                     case getAttr(string,Attrs,decode,none) of
+                         "html_entities" ->
+                             ?LOGF("The re will be decoded: ~s ~n", [Expr],?INFO),
+                             {re,Name,CompiledRegExp,fun ts_utils:conv_entities/1};
+                         _ ->
+                             {re,Name,CompiledRegExp, undefined}
+                     end;
                  xpath ->
                      ?LOGF("Add new xpath: ~s ~n", [Expr],?INFO),
                      CompiledXPathExp = mochiweb_xpath:compile_xpath(FlattenExpr),
@@ -544,8 +628,7 @@ parse(#xmlElement{name=dyn_variable, attributes=Attrs},
                      {Type,Name,Expr}
              end,
     NewDynVar = [DynVar|DynVars],
-    ?LOGF("Add new dyn variable=~p in session ~p~n",
-          [NewDynVar,CurS#session.id],?INFO),
+    ?LOGF("Add new dyn variable=~p in session ~p", [NewDynVar, CurS#session.id],?INFO),
     Conf#config{ dynvar= NewDynVar };
 
 parse( #xmlElement{name=change_type, attributes=Attrs},
@@ -553,33 +636,58 @@ parse( #xmlElement{name=change_type, attributes=Attrs},
 
     CType   = getAttr(atom, Attrs, new_type),
     Server  = getAttr(string, Attrs, host),
-    Port    = getAttr(integer, Attrs, port),
+    Port    = getAttr(string, Attrs, port),
     Store   = getAttr(atom, Attrs, store, false),
     Restore = getAttr(atom, Attrs, restore, false),
     PType   = set_net_type(getAttr(Attrs, server_type)),
+    Bidi    = getAttr(atom, Attrs, bidi, false),
     SessType=case Conf#config.main_sess_type == CType of
                  false -> CurS#session.type;
                  true  -> CType % back to the main type
              end,
-    ets:insert(Tab,{{CurS#session.id, Id+1}, {change_type, CType, Server, Port, PType, Store, Restore}}),
+    ets:insert(Tab,{{CurS#session.id, Id+1}, {change_type, CType, Server, Port, PType, Store, Restore, Bidi}}),
     ?LOGF("Parse change_type (~p) ~p:~p:~p:~p ~n",[CType, Server,Port,PType,Id],?NOTICE),
     Conf#config{main_sess_type=SessType, curid=Id+1,
                 sessions=[CurS#session{type=CType}|Other] };
 
 
+parse( #xmlElement{name=interaction, attributes=Attrs},
+      Conf = #config{sessions=[CurS|_Other], curid=Id,session_tab = Tab}) ->
+
+    Action   = list_to_atom(getAttr(string, Attrs, action, "send")),
+    RawId = getAttr(Attrs, id),
+    {ok, [{atom,1,IdInteraction}],1} = erl_scan:string("tr_"++RawId),
+
+    ets:insert(Tab,{{CurS#session.id, Id+1}, {interaction, Action, IdInteraction}}),
+    ?LOGF("Parse  interaction  ~p:~p ~n",[Action,Id],?NOTICE),
+    Conf#config{curid=Id+1 };
+
 parse( Element = #xmlElement{name=set_option, attributes=Attrs},
       Conf = #config{sessions=[CurS|_Other], curid=Id,session_tab = Tab}) ->
 
     case getAttr(atom, Attrs, type) of
         "" ->
-            {Type,Name,Args} = case getAttr(string, Attrs, name) of
-                                   "rate_limit" ->
-                                       Rate = getAttr(integer, Attrs, value),
-                                       Max  = getAttr(integer, Attrs, max, Rate),
-                                       {undefined, rate_limit, {1024*Rate div 1000, 1024 * Max}}
-                               end,
-        ets:insert(Tab,{{CurS#session.id, Id+1}, {set_option,Type,Name,Args}}),
-        Conf#config{curid=Id+1};
+            {Type,Name,Args} =
+                case getAttr(string, Attrs, name) of
+                    "rate_limit" ->
+                        Rate = getAttr(integer, Attrs, value),
+                        Max  = getAttr(integer, Attrs, max, Rate),
+                        {undefined, rate_limit, {1024*Rate div 1000, 1024 * Max}};
+                    "certificate" ->
+                        {value, #xmlElement{attributes=AttrCert}} = lists:keysearch(certificate,
+                                                                     #xmlElement.name,
+                                                                     Element#xmlElement.content),
+                        Cacert = getAttr(string, AttrCert, cacertfile, undefined),
+                        KeyFile = getAttr(string, AttrCert, keyfile, undefined),
+                        KeyPass = getAttr(string, AttrCert, keypass, undefined),
+                        CertFile = getAttr(string, AttrCert, certfile, undefined),
+                        {undefined, certificate, {Cacert, KeyFile,KeyPass,CertFile}};
+                    "connect_timeout" ->
+                        ConnectTimeout = getAttr(integer, Attrs, value),
+                        {undefined, connect_timeout, {ConnectTimeout}}
+                end,
+            ets:insert(Tab,{{CurS#session.id, Id+1}, {set_option,Type,Name,Args}}),
+            Conf#config{curid=Id+1};
         Mod ->
             Mod:parse_config(Element, Conf)
     end;
@@ -590,18 +698,28 @@ parse(Element = #xmlElement{name=request, attributes=Attrs},
 
     Type  = CurSess#session.type,
     SubstitutionFlag  = getAttr(atom, Attrs, subst, false),
-
-    lists:foldl( fun(A,B) ->Type:parse_config(A,B) end,
-                 Conf#config{curid=Id+1, cur_req_id=Id+1,
-                             subst=SubstitutionFlag,
-                             match=[]
-                            },
-                 Element#xmlElement.content);
+    Tag = getAttr(string, Attrs, tag, ""),
+    Tags = lists:map(fun(X)->{X,ok} end, string:tokens(?config(exclude_tag),",")),
+    %% do not add in Conf excluded requests
+    case proplists:is_defined(Tag, Tags) of
+        true ->
+            ?LOGF("Tag  ~p in ~p ~p ~p ~n",[Tag,true,?config(exclude_tag),Tags],?NOTICE),
+            Conf;
+        false ->
+            lists:foldl( fun(A,B) ->Type:parse_config(A,B) end,
+                         Conf#config{curid=Id+1, cur_req_id=Id+1,
+                                     subst=SubstitutionFlag,
+                                     match=[],
+                                     tag=Tag
+                                    },
+                         Element#xmlElement.content)
+        end;
 %%% Match
 parse(Element=#xmlElement{name=match,attributes=Attrs},
       Conf=#config{match=Match})->
     Do         = getAttr(atom, Attrs, do, continue),
     When       = getAttr(atom, Attrs, 'when', match),
+    Name       = getAttr(string, Attrs, name, "-"),
     Subst      = getAttr(atom, Attrs, subst, false),
     MaxLoop    = getAttr(integer, Attrs, max_loop, 20),
     LoopBack   = getAttr(integer, Attrs, loop_back, 0),
@@ -616,7 +734,7 @@ parse(Element=#xmlElement{name=match,attributes=Attrs},
                       {Mod, Fun} = ts_utils:split2(Data,$:),
                       {list_to_atom(Mod), list_to_atom(Fun)}
               end,
-    NewMatch   = #match{regexp=RegExp,subst=Subst, do=Do,'when'=When,
+    NewMatch   = #match{regexp=RegExp,subst=Subst, do=Do,'when'=When, name=Name,
                         sleep_loop=SleepLoop * 1000, skip_headers=SkipHeaders,
                         loop_back=LoopBack, max_restart=MaxRestart, max_loop=MaxLoop, apply_to_content=ApplyTo},
 
@@ -650,6 +768,17 @@ parse(Element = #xmlElement{name=option, attributes=Attrs},
                     NewProto =  OldProto#proto_opts{ssl_ciphers=Cipher},
                     lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
                                  Element#xmlElement.content);
+                "ssl_reuse_sessions" ->
+                    case getAttr(atom,Attrs, value, true) of
+                        false ->
+                            application:set_env(tsung,ssl_session_cache, 0),
+                            OldProto =  Conf#config.proto_opts,
+                            NewProto =  OldProto#proto_opts{reuse_sessions=false},
+                            lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                         Element#xmlElement.content);
+                        true -> % default value, do nothing
+                            lists:foldl( fun parse/2, Conf, Element#xmlElement.content)
+                        end;
                 "seed" ->
                     Seed =  getAttr(integer,Attrs, value, now),
                     lists:foldl( fun parse/2, Conf#config{seed=Seed},
@@ -682,6 +811,13 @@ parse(Element = #xmlElement{name=option, attributes=Attrs},
                     Port = getAttr(integer,Attrs, value, ?config(job_notify_port)),
                     lists:foldl( fun parse/2, Conf#config{job_notify_port=Port},
                                  Element#xmlElement.content);
+                "connect_timeout" ->
+                    ConnectTimeout = getAttr(integer,Attrs, value, ?config(connect_timeout)),
+                    OldProto =  Conf#config.proto_opts,
+                    NewProto =  OldProto#proto_opts{connect_timeout=ConnectTimeout},
+                    lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                 Element#xmlElement.content);
+
                 "tcp_rcv_buffer" ->
                     Size = getAttr(integer,Attrs, value, ?config(rcv_size)),
                     OldProto =  Conf#config.proto_opts,
@@ -706,16 +842,47 @@ parse(Element = #xmlElement{name=option, attributes=Attrs},
                     NewProto =  OldProto#proto_opts{udp_snd_size=Size},
                     lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
                                  Element#xmlElement.content);
-                "tcp_timeout" ->
-                    Size = getAttr(integer,Attrs, value, ?config(tcp_timeout)),
+                "idle_timeout" ->
+                    Timeout = getAttr(integer,Attrs, value, ?config(idle_timeout)),
                     OldProto =  Conf#config.proto_opts,
-                    NewProto =  OldProto#proto_opts{idle_timeout=Size},
+                    NewProto =  OldProto#proto_opts{idle_timeout=Timeout},
+                    lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                 Element#xmlElement.content);
+                "global_ack_timeout" ->
+                    Timeout = getAttr(integer,Attrs, value, ?config(global_ack_timeout)),
+                    OldProto =  Conf#config.proto_opts,
+                    NewProto =  OldProto#proto_opts{global_ack_timeout=Timeout},
+                    ts_timer:set_timeout(Timeout),
+                    lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                 Element#xmlElement.content);
+                "max_retries" ->
+                    MaxRetries = getAttr(integer,Attrs, value, ?config(max_retries)),
+                    OldProto =  Conf#config.proto_opts,
+                    NewProto =  OldProto#proto_opts{max_retries=MaxRetries},
                     lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
                                  Element#xmlElement.content);
                 "retry_timeout" ->
-                    Size = getAttr(integer,Attrs, value, ?config(client_retry_timeout)),
+                    Timeout = getAttr(integer,Attrs, value, ?config(client_retry_timeout)),
                     OldProto =  Conf#config.proto_opts,
-                    NewProto =  OldProto#proto_opts{retry_timeout=Size},
+                    NewProto =  OldProto#proto_opts{retry_timeout=Timeout},
+                    lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                 Element#xmlElement.content);
+                "websocket_path" ->
+                    Path = getAttr(string,Attrs, value, ?config(websocket_path)),
+                    OldProto =  Conf#config.proto_opts,
+                    NewProto =  OldProto#proto_opts{websocket_path=Path},
+                    lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                 Element#xmlElement.content);
+                "websocket_frame" ->
+                    Frame = getAttr(string,Attrs, value, ?config(websocket_frame)),
+                    OldProto =  Conf#config.proto_opts,
+                    NewProto =  OldProto#proto_opts{websocket_frame=Frame},
+                    lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                 Element#xmlElement.content);
+                "bosh_path" ->
+                    Path = getAttr(string,Attrs, value, ?config(bosh_path)),
+                    OldProto =  Conf#config.proto_opts,
+                    NewProto =  OldProto#proto_opts{bosh_path=Path},
                     lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
                                  Element#xmlElement.content);
                 "file_server" ->
@@ -730,6 +897,21 @@ parse(Element = #xmlElement{name=option, attributes=Attrs},
                     lists:foldl( fun parse/2,
                                  Conf#config{file_server=[{Id, FileName} | Conf#config.file_server]},
                                  Element#xmlElement.content);
+                "global_number" ->
+                    GlobalNumber = getAttr(integer, Attrs, value, ?config(global_number)),
+                    ts_timer:config(GlobalNumber),
+                    lists:foldl( fun parse/2, Conf, Element#xmlElement.content);
+                "tcp_reuseaddr" ->
+                    Reuseaddr = getAttr(atom, Attrs, value, false),
+                    case Reuseaddr of
+                        true ->
+                            OldProto =  Conf#config.proto_opts,
+                            NewProto =  OldProto#proto_opts{tcp_reuseaddr = Reuseaddr},
+                            lists:foldl( fun parse/2, Conf#config{proto_opts=NewProto},
+                                         Element#xmlElement.content);
+                        false ->
+                            lists:foldl( fun parse/2, Conf, Element#xmlElement.content)
+                    end;
                 Other ->
                     ?LOGF("Unknown option ~p !~n",[Other], ?WARN),
                     lists:foldl( fun parse/2, Conf, Element#xmlElement.content)
@@ -744,6 +926,8 @@ parse(Element = #xmlElement{name=option, attributes=Attrs},
 parse(Element = #xmlElement{name=thinktime, attributes=Attrs},
       Conf = #config{curid=Id, session_tab = Tab, sessions = [CurS |_]}) ->
     {RT,T} = case getAttr(Attrs, value)  of
+            "wait_bidi" ->
+                {infinity, infinity};
             "wait_global" ->
                 {wait_global,infinity};
             "%%"++Tail -> % dynamic thinktime
@@ -806,7 +990,7 @@ parse(Element = #xmlElement{name=setdynvars, attributes=Attrs},
                      FileId = getAttr(atom,Attrs,fileid,none),
                      case lists:keysearch(FileId,1,Conf#config.file_server) of
                          {value,_Val} ->
-                             Delimiter = getAttr(string,Attrs,delimiter,";"),
+                             Delimiter = list_to_binary(getAttr(string,Attrs,delimiter,";")),
                              {setdynvars,file,{Order,FileId,Delimiter},Vars};
                          false ->
                              io:format(standard_error, "Unknown_file_id ~p in file setdynvars declaration: you forgot to add a file_server option~n",[FileId]),
@@ -825,7 +1009,12 @@ parse(Element = #xmlElement{name=setdynvars, attributes=Attrs},
                  "jsonpath" ->
                      From = getAttr(atom, Attrs,from),
                      JSONPath = getAttr(Attrs,jsonpath),
-                     {setdynvars,jsonpath,{JSONPath, From},Vars}
+                     {setdynvars,jsonpath,{JSONPath, From},Vars};
+                 "value" ->
+                     Value = getAttr(string,Attrs,value,""),
+                     {setdynvars,value,{string,Value},Vars};
+                 "server" ->
+                     {setdynvars,server,{},Vars}
              end,
     ?LOGF("Add setdynvars in session ~p as id ~p",[CurS#session.id,Id+1],?INFO),
     ets:insert(Tab, {{CurS#session.id, Id+1}, Action}),
@@ -915,6 +1104,9 @@ get_default(Tab, Key,ConfigName) ->
             SName
     end.
 
+get_default(Tab, Key) when is_atom(Key) ->
+    get_default(Tab, Key, Key).
+
 %%%----------------------------------------------------------------------
 %%% Function: mark_prev_req/3
 %%% Purpose: use to set page marks in requests during parsing ; by
@@ -994,12 +1186,16 @@ read_stdio(eof, Data)->
 read_stdio(Data,Acc) ->
     read_stdio(io:get_line(""),[Acc,Data]).
 
-set_net_type("tcp")  -> gen_tcp;
-set_net_type("tcp6") -> gen_tcp6;
-set_net_type("udp")  -> gen_udp;
-set_net_type("udp6") -> gen_udp6;
-set_net_type("ssl")  -> ssl;
-set_net_type("ssl6") -> ssl6.
+set_net_type("tcp")   -> ts_tcp;
+set_net_type("tcp6")  -> ts_tcp6;
+set_net_type("udp")   -> ts_udp;
+set_net_type("udp6")  -> ts_udp6;
+set_net_type("ssl")   -> ts_ssl;
+set_net_type("ssl6")  -> ts_ssl6;
+set_net_type("websocket")  -> ts_server_websocket;
+set_net_type("bosh")  -> ts_bosh;
+set_net_type("bosh_ssl") -> ts_bosh_ssl;
+set_net_type("erlang") -> ts_erlang.
 
 get_dynvar_name(VarNameStr) ->
     %% check if the var name is for an array (myvar[N])
@@ -1007,3 +1203,24 @@ get_dynvar_name(VarNameStr) ->
         {match,[Name,Index]} -> {list_to_atom(Name),Index};
         _                    -> list_to_atom(VarNameStr)
     end.
+
+
+%% @spec get_popularity(Proba::number(), Weight::number(), UseWeight::(true|false|undefined), Total::number()) ->
+%%   {Value::number(), UseWeight::boolean(),  Total::number()}
+%% @doc check if we are using popularity or weights; keep the total up to date. @end
+get_popularity(-1, -1, _, _)->
+    erlang:error({"must set weight or probability in session"});
+get_popularity(Proba,Weight,_,_) when is_number(Proba), Proba >= 0, is_number(Weight), Weight >= 0 ->
+    erlang:error({"can't mix probabilites and weights", Proba, Weight} );
+get_popularity(Proba, _Weight, true,_)     when is_number(Proba), Proba >= 0->
+    erlang:error({"can't use probability when using weight"});
+get_popularity(_, Weight, false,_)        when is_number(Weight), Weight >= 0->
+    erlang:error({"can't use weights when using probabilities"});
+get_popularity(_, Weight, undefined,_)    when is_number(Weight), Weight >= 0 ->
+    {Weight, true, Weight};
+get_popularity(Proba, _, undefined,Total) when is_number(Proba) ->
+    {Proba, false, Proba+Total};
+get_popularity(Proba, _, false,Total) when is_number(Proba) ->
+    {Proba, false, Proba+Total};
+get_popularity(_, Weight, true, Total)    when is_number(Weight) ->
+    {Weight, true, Weight+Total}.
