@@ -285,7 +285,7 @@ handle_call({get_next_session, HostName, PhaseId}, _From, State=#state{users=Use
     ?DebugF("get new session for ~p~n",[_From]),
     case choose_session(Config#config.sessions, Config#config.total_popularity, PhaseId) of
         {ok, Session=#session{id=Id}} ->
-            ?LOGF("Session ~p choosen~n",[Id],?INFO),
+            ?LOGF("Session ~p chosen~n",[Id],?INFO),
             ts_mon:newclient({Id,?NOW}),
             {IPParam, Server} = get_user_param(Client,Config),
             {reply, {ok, Session#session{client_ip= IPParam, server=Server,userid=Users,
@@ -334,10 +334,10 @@ handle_call({get_monitor_hosts}, _From, State) ->
     Config = State#state.config,
     {reply, Config#config.monitor_hosts, State};
 
-% get status: send the number of actives nodes
+% get status: send the number of actives nodes, number of phases
 handle_call({status}, _From, State) ->
     Config = State#state.config,
-    Reply = {ok, length(Config#config.clients), State#state.ending_beams},
+    Reply = {ok, length(Config#config.clients), State#state.ending_beams, length(Config#config.arrivalphases) },
     {reply, Reply, State};
 
 handle_call({get_jobs_state}, _From, State) when State#state.config == undefined ->
@@ -379,7 +379,11 @@ handle_cast({newbeams, HostList}, State=#state{logdir   = LogDir,
             Args = set_remote_args(LogDir, Config#config.ports_range),
             {BeamsIds, LastId} = lists:mapfoldl(fun(A,Acc) -> {{A, Acc}, Acc+1} end, Id0, RemoteBeams),
             Fun = fun({Host,Id}) -> remote_launcher(Host, Id, Args) end,
-            RemoteNodes = ts_utils:pmap(Fun, BeamsIds),
+            %% start beams in parallel, at most 10 in parallel
+            %% (because sshd MaxSessions = 10, in case we have to
+            %% start more than 10 beams on a single host)
+            RemoteNodes = ts_utils:pmap(Fun, BeamsIds,9),
+            check_remotes_ok(RemoteNodes),
             ?LOG("All remote beams started, syncing ~n",?NOTICE),
             global:sync(),
             ?LOG("Syncing done, start remote tsung application ~n", ?DEB),
@@ -428,13 +432,20 @@ handle_cast({newbeam, Host, _}, State=#state{ hostname=LocalHost,config=Config})
     {noreply, State};
 
 %% start a launcher on a new beam with slave module
-handle_cast({newbeam, Host, Args}, State=#state{last_beam_id = NodeId, config=Config, logdir = LogDir}) ->
+handle_cast({newbeam, Host, Arrivals}, State=#state{last_beam_id = NodeId, config=Config, logdir = LogDir}) ->
     Args = set_remote_args(LogDir,Config#config.ports_range),
     Seed = Config#config.seed,
     Node = remote_launcher(Host, NodeId, Args),
-    ts_launcher_static:stop(Node), % no need for static launcher in this case (already have one)
-    ts_launcher:launch({Node, Args, Seed}),
-    {noreply, State#state{last_beam_id = NodeId+1}};
+    case rpc:call(Node,tsung,start,[],?RPC_TIMEOUT) of
+        {badrpc, Reason} ->
+            ?LOGF("Fail to start tsung on beam ~p, reason: ~p",[Node,Reason], ?ERR),
+            slave:stop(Node),
+            {noreply, State};
+        _ ->
+            ts_launcher_static:stop(Node), % no need for static launcher in this case (already have one)
+            ts_launcher:launch({Node, Arrivals, Seed}),
+            {noreply, State#state{last_beam_id = NodeId+1}}
+    end;
 
 handle_cast({end_launching, _Node}, State=#state{ending_beams=Beams}) ->
     {noreply, State#state{ending_beams = Beams+1}};
@@ -669,7 +680,7 @@ setup_user_servers(FileId,Val) when is_atom(FileId), is_integer(Val) ->
     {ok,Domains} = ts_file_server:get_all_lines(FileId),
     ?LOGF("Domains:~p~n",[Domains],?DEB),
     lists:foreach(fun(Domain) ->
-                    {ok,_} = ts_user_server_sup:start_user_server(list_to_atom("us_" ++Domain))
+                    {ok,_} = ts_user_server_sup:start_user_server(list_to_atom("us_" ++binary_to_list(Domain)))
                   end, Domains),
     ts_user_server:reset_all(Val).
 
@@ -760,7 +771,7 @@ start_slave(Host, Name, Args) when is_atom(Host), is_atom(Name)->
             Node;
         {error, Reason} ->
             ?LOGF("Can't start newbeam on host ~p (reason: ~p) ! Aborting!~n",[Host, Reason],?EMERG),
-            exit({slave_failure, Reason})
+            {error, {slave_failure, Reason}}
     end.
 choose_port(_,_, undefined) ->
     {[],0};
@@ -857,6 +868,14 @@ remote_launcher(Host, NodeId, Args)->
     ?LOGF("starting newbeam ~p on host ~p with Args ~p~n", [Name, Host, Args], ?INFO),
     start_slave(Host, Name, Args).
 
+check_remotes_ok(Remotes) ->
+    lists:foreach(fun({error, Reason}) ->
+                          ts_mon:abort(),
+                          exit(Reason);
+                     (_) ->
+                          ok
+                  end, Remotes).
+
 set_remote_args(LogDir,PortsRange)->
     {ok, PAList}    = init:get_argument(pa),
     PA = lists:flatmap(fun(A) -> [" -pa "] ++A end,PAList),
@@ -930,7 +949,7 @@ update_total_pop(UseWeight,Phases, Sessions) ->
     update_total_pop(UseWeight, length(Phases), Sessions, []).
 
 update_total_pop(_UseWeight,0, _, Total) ->
-    ?LOGF("New Total popularities:~p",[Total],?DEB),
+    ?LOGF("New Total popularities:~w",[Total],?DEB),
     Total;
 update_total_pop(UseWeight,N, Sessions, Total)   ->
     Sum = fun(#session{popularity=P},Acc) when is_number(P) ->
@@ -942,6 +961,8 @@ update_total_pop(UseWeight,N, Sessions, Total)   ->
     update_total_pop(UseWeight, N-1, Sessions, [PhaseTotal |Total]).
 
 %% set popularity of session 'Name' per phase (needed when <session_setup> is used)
+set_pop(_Name,Popularity,[]) ->
+    Popularity;
 set_pop(Name,Popularity,Phases) ->
     set_pop(Name,Popularity,Phases,[]).
 
