@@ -19,7 +19,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 
 %%----------------------------------------------------------------------
@@ -37,7 +38,6 @@
 
 -behaviour(gen_server).
 
--include("ts_profile.hrl").
 -include("ts_config.hrl").
 
 %% External exports
@@ -55,10 +55,13 @@
 -define(DELAYED_WRITE_SIZE,524288). % 512KB
 -define(DELAYED_WRITE_DELAY,5000).  % 5 sec
 
+%% one global ts_stats_mon procs + 4 dedicated stats procs to share the load
+-define(STATSPROCS, [request, connect, page, transaction, ts_stats_mon]).
+
 -record(state, {log,          % log fd
                 backend,      % type of backend: text|...
                 log_dir,      % log directory
-                fullstats,    % fullstats log filename
+                fullstats,    % fullstats fd
                 dump_interval,%
                 dumpfile,     % file used when dumptrafic is set light or full
                 client=0,     % number of clients currently running
@@ -68,7 +71,9 @@
                 laststats,    % values of last printed stats
                 lastdate,     % date of last printed stats
                 type,         % type of logging (none, light, full)
-                launchers=0   % number of launchers started
+                launchers=0,  % number of launchers started
+                timer_ref,    % timer reference (for dumpstats)
+                wait_gui=false% wait gui before stopping
                }).
 
 -record(stats, {
@@ -106,12 +111,12 @@ add(nocache,Data) ->
 add(Data) ->
     ts_mon_cache:add(Data).
 
-add_match(Data,{UserId,SessionId,RequestId}) ->
-    add_match(Data,{UserId,SessionId,RequestId,[]});
-add_match(Data=[Head|_],{UserId,SessionId,RequestId,Bin}) ->
-    TimeStamp=now(),
+add_match(Data,{UserId,SessionId,RequestId,Tr,Name}) ->
+    add_match(Data,{UserId,SessionId,RequestId,[],Tr,Name});
+add_match(Data=[Head|_],{UserId,SessionId,RequestId,Bin,Tr,Name}) ->
+    TimeStamp=?NOW,
     put(last_match,Head),
-    ts_mon_cache:add_match(Data,{UserId,SessionId,RequestId,TimeStamp, Bin}).
+    ts_mon_cache:add_match(Data,{UserId,SessionId,RequestId,TimeStamp, Bin,Tr,Name}).
 
 status() ->
     gen_server:call({global, ?MODULE}, {status}).
@@ -130,18 +135,23 @@ endclient({Who, When, Elapsed}) ->
 
 sendmes({none, _, _})       -> skip;
 sendmes({protocol, _, _})   -> skip;
+sendmes({protocol_local, _, _})   -> skip;
 sendmes({_Type, Who, What}) ->
-    gen_server:cast({global, ?MODULE}, {sendmsg, Who, now(), What}).
+    gen_server:cast({global, ?MODULE}, {sendmsg, Who, ?NOW, What}).
 
 rcvmes({none, _, _})    -> skip;
 rcvmes({protocol, _, _})-> skip;
+rcvmes({protocol_local, _, _})-> skip;
 rcvmes({_, _, closed})  -> skip;
 rcvmes({_Type, Who, What})  ->
-    gen_server:cast({global, ?MODULE}, {rcvmsg, Who, now(), What}).
+    gen_server:cast({global, ?MODULE}, {rcvmsg, Who, ?NOW, What}).
 
 dump({none, _, _})-> skip;
-dump({Type, Who, What})  ->
-    gen_server:cast({global, ?MODULE}, {dump, Who, now(), What}).
+dump({cached, << >> })-> skip;
+dump({_Type, Who, What})  ->
+    gen_server:cast({global, ?MODULE}, {dump, Who, ?NOW, What});
+dump({cached, Data})->
+    gen_server:cast({global, ?MODULE}, {dump, cached, Data}).
 
 launcher_is_alive() ->
     gen_server:cast({global, ?MODULE}, {launcher_is_alive}).
@@ -159,23 +169,27 @@ launcher_is_alive() ->
 %%          {stop, Reason}
 %%----------------------------------------------------------------------
 init([LogDir]) ->
-    ?LOGF("Init, log dir is ~p~n",[LogDir],?NOTICE),
-    Base = filename:basename(?config(log_file)),
-    Filename = filename:join(LogDir, Base),
-    case file:open(Filename,[write]) of
-        {ok, Stream} ->
-            ?LOG("starting monitor~n",?NOTICE),
-            Stats = #stats{os_mon  = dict:new()},
-            {ok, #state{ log       = Stream,
-                         dump_interval = ?config(dumpstats_interval),
-                         log_dir   = LogDir,
-                         stats     = Stats,
-                         lastdate  = now(),
-                         laststats = Stats
-                       }};
-        {error, Reason} ->
-            ?LOGF("Can't open mon log file! ~p~n",[Reason], ?ERR),
-            {stop, Reason}
+    ?LOGF("Init, log dir is ~p~n",[LogDir],?INFO),
+    Stats = #stats{os_mon  = dict:new()},
+    State=#state{ dump_interval = ?config(dumpstats_interval),
+                  log_dir   = LogDir,
+                  stats     = Stats,
+                  lastdate  = ?NOW,
+                  laststats = Stats
+                },
+    case ?config(mon_file) of
+        "-" ->
+            {ok, State#state{log=standard_io}};
+        Name ->
+            Filename = filename:join(LogDir, Name),
+            case file:open(Filename,[write]) of
+                {ok, Stream} ->
+                    ?LOG("starting monitor~n",?INFO),
+                    {ok, State#state{log=Stream}};
+                {error, Reason} ->
+                    ?LOGF("Can't open mon log file! ~p~n",[Reason], ?ERR),
+                    {stop, Reason}
+            end
     end.
 
 %%----------------------------------------------------------------------
@@ -191,15 +205,22 @@ handle_call({start_logger, Machines, DumpType, Backend}, From, State) ->
     start_logger({Machines, DumpType, Backend}, From, State);
 
 %%% get status
-handle_call({status}, _From, State) ->
-    Request   = ts_stats_mon:status(request),
-    Interval  = ts_utils:elapsed(State#state.lastdate, now()) / 1000,
-    Phase     = ts_stats_mon:status(newphase,sum),
-    Connected =  case ts_stats_mon:status(connected,sum) of
+handle_call({status}, _From, State=#state{stats=Stats}) ->
+    {ok, Localhost} = ts_utils:node_to_hostname(node()),
+    CpuName         = {{cpu,"tsung_controller@"++Localhost}, sample},
+    CPU = case dict:find(CpuName,Stats#stats.os_mon) of
+              {ok, [ValCPU|_]} -> ValCPU ;
+              _ -> 0
+          end,
+
+    Request     = ts_stats_mon:status(request),
+    Interval    = ts_utils:elapsed(State#state.lastdate, ?NOW) / 1000,
+    Phase       = ts_stats_mon:status(newphase,sum),
+    Connected   =  case ts_stats_mon:status(connected,sum) of
                      {ok, Val} -> Val;
                      _ -> 0
                  end,
-    Reply = { State#state.client, Request,Connected, Interval, Phase},
+    Reply = { State#state.client, Request, Connected, Interval, Phase, CPU},
     {reply, Reply, State};
 
 handle_call(Request, _From, State) ->
@@ -213,6 +234,8 @@ handle_call(Request, _From, State) ->
 %%          {noreply, State, Timeout} |
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
+handle_cast({add, _Data}, State=#state{wait_gui=true}) ->
+    {noreply,State};
 handle_cast({add, Data}, State=#state{stats=Stats}) when is_list(Data) ->
     case State#state.backend of
         fullstats -> io:format(State#state.fullstats,"~p~n",[Data]);
@@ -239,6 +262,7 @@ handle_cast({newclient, Who, When}, State=#state{stats=Stats}) ->
     case State#state.type of
         none     -> ok;
         protocol -> ok;
+        protocol_local -> ok;
         _ ->
             io:format(State#state.dumpfile,"NewClient:~w:~p~n",[ts_utils:time2sec_hires(When), Who]),
             io:format(State#state.dumpfile,"load:~w~n",[Clients])
@@ -263,24 +287,20 @@ handle_cast({endclient, Who, When, Elapsed}, State=#state{stats=Stats}) ->
             skip;
         protocol ->
             skip;
+        protocol_local ->
+            skip;
         _Type ->
             io:format(State#state.dumpfile,"EndClient:~w:~p~n",[ts_utils:time2sec_hires(When), Who]),
             io:format(State#state.dumpfile,"load:~w~n",[Clients])
     end,
-    case {Clients, State#state.stop} of
-        {0, true} ->
-            ?LOG("No more users and stop is true, stop~n", ?INFO),
-            {stop, normal, State};
-        _ ->
-            {noreply, State#state{client = Clients, stats=NewStats}}
-    end;
+    {noreply, State#state{client = Clients, stats=NewStats}};
 
 handle_cast({dumpstats}, State=#state{stats=Stats}) ->
     export_stats(State),
     NewSessions = ts_stats_mon:reset_all_stats(Stats#stats.session),
     NewOSmon = ts_stats_mon:reset_all_stats(Stats#stats.os_mon),
     NewStats = Stats#stats{session=NewSessions, os_mon=NewOSmon},
-    {noreply, State#state{laststats = Stats, stats=NewStats,lastdate=now()}};
+    {noreply, State#state{laststats = Stats, stats=NewStats,lastdate=?NOW}};
 
 
 handle_cast({sendmsg, _, _, _}, State = #state{type = none}) ->
@@ -302,6 +322,10 @@ handle_cast({dump, Who, When, What}, State=#state{type=protocol,dumpfile=Log}) -
     io:format(Log,"~w;~w;~s~n",[ts_utils:time2sec_hires(When),Who,What]),
     {noreply, State};
 
+handle_cast({dump, cached, Data}, State=#state{type=protocol,dumpfile=Log}) ->
+    file:write(Log,Data),
+    {noreply, State};
+
 handle_cast({rcvmsg, _, _, _}, State = #state{type=none}) ->
     {noreply, State};
 
@@ -320,15 +344,23 @@ handle_cast({rcvmsg, Who, When, What}, State=#state{type=full,dumpfile=Log}) ->
     io:format(Log, "Recv:~w:~w:~p~n",[ts_utils:time2sec_hires(When),Who,What]),
     {noreply, State};
 
-handle_cast({stop}, State = #state{client = 0, launchers=1}) ->
+handle_cast({stop}, State = #state{client=0, launchers=1, timer_ref=TRef}) ->
     ?LOG("Stop asked, no more users, last launcher stopped, OK to stop~n", ?INFO),
-    {stop, normal, State};
+    case ?config(keep_web_gui) of
+        true ->
+            io:format(standard_io,"All slaves have stopped; keep controller and web dashboard alive. ~nHit CTRL-C or click Stop on the dashboard to stop.~n",[]),
+            timer:cancel(TRef),
+            close_stats(State),
+            {noreply, State#state{wait_gui=true}};
+        _ ->
+            {stop, normal, State}
+    end;
 handle_cast({stop}, State=#state{launchers=L}) -> % we should stop, wait until no more clients are alive
     ?LOG("A launcher has finished, but not all users have finished, wait before stopping~n", ?NOTICE),
     {noreply, State#state{stop = true, launchers=L-1}};
 
 handle_cast({launcher_is_alive}, State=#state{launchers=L}) ->
-    ?LOG("A launcher has started~n", ?NOTICE),
+    ?LOG("A launcher has started~n", ?INFO),
     {noreply, State#state{launchers=L+1}};
 
 
@@ -351,21 +383,35 @@ handle_cast(Msg, State) ->
 handle_info(_Info, State) ->
     {noreply, State}.
 
+
+close_stats(State) ->
+    export_stats(State),
+    % blocking call to all ts_stats_mon; this way, we are
+    % sure the last call to dumpstats is finished
+    lists:foreach(fun(Name) -> ts_stats_mon:status(Name) end, ?STATSPROCS),
+    case State#state.backend of
+        json ->
+            io:format(State#state.log,"]}]}~n",[]);
+        _ ->
+            io:format(State#state.log,"EndMonitor:~w~n",[?NOW])
+        end,
+    case State#state.log of
+        standard_io -> ok;
+        Dev         -> file:close(Dev)
+    end,
+    file:close(State#state.fullstats).
+
 %%----------------------------------------------------------------------
 %% Func: terminate/2
 %% Purpose: Shutdown the server
 %% Returns: any (ignored by gen_server)
 %%----------------------------------------------------------------------
+terminate(Reason, #state{wait_gui=true}) ->
+    ?LOGF("stopping monitor by gui (~p)~n",[Reason],?NOTICE);
 terminate(Reason, State) ->
     ?LOGF("stopping monitor (~p)~n",[Reason],?NOTICE),
-    export_stats(State),
-    ts_stats_mon:status(ts_stats_mon), % blocking call to ts_stats_mon; this way, we are
-                                       % sure the last call to dumpstats is finished
-    io:format(State#state.log,"EndMonitor:~w~n",[now()]),
-    file:close(State#state.log),
-    file:close(State#state.fullstats),
-    slave:stop(node()),
-    ok.
+    close_stats(State),
+    slave:stop(node()).
 
 %%--------------------------------------------------------------------
 %% Func: code_change/3
@@ -385,35 +431,30 @@ code_change(_OldVsn, StateData, _Extra) ->
 %% Returns: {reply, ok, State} | {stop, Reason, State}
 %%----------------------------------------------------------------------
 %% fulltext backend: open log file with compression enable and delayed_write
-start_logger({Machines, DumpType, fullstats}, _From, State) ->
+start_logger({Machines, DumpType, fullstats}, From, State=#state{fullstats=undefined}) ->
     Filename = filename:join(State#state.log_dir,?FULLSTATS_FILENAME),
     ?LOG("Open file with delayed_write for fullstats backend~n",?NOTICE),
     case file:open(Filename,[write, {delayed_write, ?DELAYED_WRITE_SIZE, ?DELAYED_WRITE_DELAY}]) of
         {ok, Stream} ->
-            ?LOG("Activate clients with fullstats backend~n",?NOTICE),
-            timer:apply_interval(State#state.dump_interval, ?MODULE, dumpstats, [] ),
-            start_launchers(Machines),
-            ts_stats_mon:set_output(fullstats,{State#state.log, Stream}),
-            ts_stats_mon:set_output(fullstats,{State#state.log, Stream}, transaction),
-            ts_stats_mon:set_output(fullstats,{State#state.log, Stream}, request),
-            ts_stats_mon:set_output(fullstats,{State#state.log, Stream}, connect),
-            ts_stats_mon:set_output(fullstats,{State#state.log, Stream}, page),
-            start_dump(State#state{type=DumpType, backend=fullstats, fullstats=Stream});
+            start_logger({Machines, DumpType, fullstats}, From, State#state{fullstats=Stream});
         {error, Reason} ->
             ?LOGF("Can't open mon log file ~p! ~p~n",[Filename,Reason], ?ERR),
             {stop, Reason, State}
     end;
 
-start_logger({Machines, DumpType, Backend}, _From, State) ->
+start_logger({Machines, DumpType, Backend}, _From, State=#state{log=Log,fullstats=FS}) ->
     ?LOGF("Activate clients with ~p backend~n",[Backend],?NOTICE),
-    timer:apply_interval(State#state.dump_interval, ?MODULE, dumpstats, [] ),
+    print_headline(Log,Backend),
     start_launchers(Machines),
-    ts_stats_mon:set_output(text,{State#state.log,[]}),
-    ts_stats_mon:set_output(text,{State#state.log,[]}, transaction),
-    ts_stats_mon:set_output(text,{State#state.log,[]}, request),
-    ts_stats_mon:set_output(text,{State#state.log,[]}, connect),
-    ts_stats_mon:set_output(text,{State#state.log,[]}, page),
-    start_dump(State#state{type=DumpType, backend=text}).
+    {ok, TRef} = timer:apply_interval(State#state.dump_interval, ?MODULE, dumpstats, [] ),
+    lists:foreach(fun(Name) -> ts_stats_mon:set_output(Backend,{Log,FS}, Name) end, ?STATSPROCS),
+    start_dump(State#state{type=DumpType, backend=Backend, timer_ref=TRef}).
+
+print_headline(Log,json)->
+    DateStr = ts_utils:now_sec(),
+    io:format(Log,"{~n \"stats\": [~n {\"timestamp\": ~p,  \"samples\": [",[DateStr]);
+print_headline(_Log,_Backend)->
+    ok.
 
 %% @spec start_dump(State::record(state)) -> {reply, Reply, State}
 %% @doc open file for dumping traffic
@@ -426,7 +467,7 @@ start_dump(State=#state{type=Type}) ->
             ?LOG("dump file opened, starting monitor~n",?INFO),
             case Type of
                 protocol ->
-                    io:format(Stream,"#date;pid;id;http method;host;URL;HTTP status;size;match;error~n",[]);
+                    io:format(Stream,"#date;pid;id;http method;host;URL;HTTP status;size;duration;transaction;match;error;tag~n",[]);
                 _ ->
                     ok
             end,
@@ -439,35 +480,70 @@ start_dump(State=#state{type=Type}) ->
 %%----------------------------------------------------------------------
 %% Func: export_stats/1
 %%----------------------------------------------------------------------
-export_stats(State=#state{log=Log,stats=Stats,laststats=LastStats}) ->
+export_stats(State=#state{log=Log,stats=Stats,laststats=LastStats, backend=json}) ->
+    DateStr = ts_utils:now_sec(),
+    io:format(Log,"]},~n {\"timestamp\": ~w,  \"samples\": [",[DateStr]),
+    %% print number of simultaneous users
+    io:format(Log,"   {\"name\": \"users\", \"value\": ~p, \"max\": ~p}",[State#state.client,State#state.maxclient]),
+    export_stats_common(json, Stats,LastStats,Log);
+
+export_stats(State=#state{log=Log,stats=Stats,laststats=LastStats, backend=BackEnd}) ->
     DateStr = ts_utils:now_sec(),
     io:format(Log,"# stats: dump at ~w~n",[DateStr]),
     %% print number of simultaneous users
     io:format(Log,"stats: ~p ~p ~p~n",[users,State#state.client,State#state.maxclient]),
-    Param = {(State#state.laststats)#stats.os_mon,State#state.log},
-    dict:fold(fun ts_stats_mon:print_stats_txt/3, Param, (State#state.stats)#stats.os_mon),
-    ts_stats_mon:print_stats_txt({session, sample}, Stats#stats.session,{[],Log}),
-    ts_stats_mon:print_stats_txt({users_count, count},
+    export_stats_common(BackEnd, Stats,LastStats,Log).
+
+export_stats_common(BackEnd, Stats,LastStats,Log)->
+    Param = {BackEnd,LastStats#stats.os_mon,Log},
+    dict:fold(fun ts_stats_mon:print_stats/3, Param, Stats#stats.os_mon),
+    ts_stats_mon:print_stats({session, sample}, Stats#stats.session,{BackEnd,[],Log}),
+    ts_stats_mon:print_stats({users_count, count},
                                  Stats#stats.users_count,
-                                 {LastStats#stats.users_count,Log}),
-    ts_stats_mon:print_stats_txt({finish_users_count, count},
-                                 Stats#stats.finish_users_count,
-                                 {LastStats#stats.finish_users_count,Log}),
-    ts_stats_mon:dumpstats(request),
-    ts_stats_mon:dumpstats(page),
-    ts_stats_mon:dumpstats(connect),
-    ts_stats_mon:dumpstats(transaction),
-    ts_stats_mon:dumpstats().
+                                 {BackEnd,LastStats#stats.users_count,Log}),
+    ts_stats_mon:print_stats({finish_users_count, count},
+                             Stats#stats.finish_users_count,
+                             {BackEnd,LastStats#stats.finish_users_count,Log}),
+    lists:foreach(fun(Name) -> ts_stats_mon:dumpstats(Name) end, ?STATSPROCS).
 
 %%----------------------------------------------------------------------
 %% Func: start_launchers/2
 %% @doc start the launcher on clients nodes
 %%----------------------------------------------------------------------
 start_launchers(Machines) ->
-    ?DebugF("Need to start tsung client on ~p~n",[Machines]),
+    ?LOGF("Tsung clients setup: ~p~n",[Machines],?DEB),
     GetHost = fun(A) -> list_to_atom(A#client.host) end,
     HostList = lists:map(GetHost, Machines),
-    ?DebugF("Hostlist is ~p~n",[HostList]),
+    ?LOGF("Starting tsung clients on hosts: ~p~n",[HostList],?NOTICE),
     %% starts beam on all client hosts
     ts_config_server:newbeams(HostList).
 
+%% post_process_logs(FileName) ->
+%%     {ok, Device} = file:open(FileName, [read]),
+%%     post_process_line(io:get_line(Device, ""), Device, []).
+
+%% post_process_line(eof, Device, State) ->
+%%     file:close(Device);
+%% post_process_line("End "++ _, Device, Logs) ->
+%%     post_process_line(io:get_line(Device, ""),Device, Logs);
+%% post_process_line("# stats: dump at "++ TimeStamp, D, Logs=#logs{start_time=undefined}) ->
+%%     {StartTime,_}=string:to_integer(TimeStamp),
+%%     post_process_line(io:get_line(D, ""),D, #logs{start_time=StartTime});
+%% post_process_line("# stats: dump at "++ TimeStamp, Dev, Logs) ->
+%%     {Time,_}=string:to_integer(TimeStamp),
+%%     Current=Time-Logs#logs.start_time,
+%%     post_process_line(io:get_line(Dev, ""),Dev, Logs#logs{current_time=Current});
+%% post_process_line("# stats:  "++ Stats, Dev, Logs) ->
+%%     case string:tokens(Stats," ") of
+%%         {"users",Count, GlobalCount} ->
+%%             todo;
+%%         {Name, Count, Max} ->
+%%             todo;
+%%         {"tr_" ++ TrName, Count, Mean, StdDev, Max, Min, GMean,GCount} ->
+%%             todo;
+%%         {"{"++ Name, Count, Mean, StdDev, Max, Min, GMean,GCount} ->
+%%             todo;
+%%         {Name, Count, Mean, StdDev, Max, Min, GMean,GCount} ->
+%%             todo
+%%     end,
+%%     post_process_line(io:get_line(Dev, ""),Dev, Logs).
